MONSTER( 'MOIP.Components.Settings', function(Model, $, utils) {

	var errorClass = utils.addPrefix( 'field-error' );

	Model.fn.start = function() {
		this.init();
	};

	Model.fn.init = function() {
		this.publicKey    = $( '[data-field="public-key"]' );
		this.installments = $( '[data-field="installments"]' );
		this.billet       = $( '[data-field="billet"]' );

		this.handleElementsVisibility( this.elements.checkout.val() );
		this.hideInstallments( $( '[data-action=installments-maximum]' ).val() );
		this.addEventListener();
	};

	Model.fn.addEventListener = function() {
		this.on( 'keyup', 'invoice-name' );
		this.on( 'change', 'checkout-type' );
		this.on( 'change', 'installments-maximum' );

		$( '#oauth-app-btn' ).on( 'click', this._onClickOauthApp.bind( this ) );
		$( '#mainform' ).on( 'submit', this._onSubmitForm.bind( this ) );
	};

	Model.fn._onKeyupInvoiceName = function(event) {
		if ( event.currentTarget.value.length > 13 ) {
			$( event.currentTarget ).addClass( errorClass );
			return;
		}

		$( event.currentTarget ).removeClass( errorClass );
	};

	Model.fn._onSubmitForm = function(event) {
		this.toTop = false;
		this.items = [];

		this.elements.validate.each( this._eachValidate.bind( this ) );

		return !~this.items.indexOf( true );
	};

	Model.fn._onChangeCheckoutType = function(event) {
		this.handleElementsVisibility( event.currentTarget.value );
	};

	Model.fn._onChangeInstallmentsMaximum = function(event) {
		this.hideInstallments( event.currentTarget.value );
	};

	Model.fn.hideInstallments = function(max) {
		var installments = $( '[data-installment]' );

		installments.each(function(index, item) {
			var installment = $( item );
			if ( parseInt( item.dataset.installment ) > parseInt( max ) ) {
				installment.hide();
			} else {
				installments.show();
			}
		});
	};

	Model.fn._onClickOauthApp = function(event) {
		this.body.find( '#app-overlay' ).slideToggle();
	};

	Model.fn._eachValidate = function(index, field) {
		var rect;
		var element = $( field )
		  , empty   = element.isEmptyValue()
		  , func    = empty ? 'addClass' : 'removeClass'
		;

		if ( !element.is( ':visible' ) ) {
			return;
		}

		element[func]( errorClass );

		this.items[index] = empty;

		if ( !empty ) {
			return;
		}

		field.placeholder = field.dataset.errorMsg;

		if ( !this.toTop ) {
			this.toTop = true;
			rect       = field.getBoundingClientRect();
			window.scrollTo( 0, ( rect.top + window.scrollY ) - 40 );
		}
	};

	Model.fn.handleElementsVisibility = function(checkoutValue) {
		var publicKeyContainer 	  = this.publicKey.closest( 'tr' )
		  , installmentsContainer = this.installments.closest( '.form-table' )
		  , installmentsTitle 	  = installmentsContainer.prev()
		  , billetContainer 	  = this.billet.closest( '.form-table' )
		  , billetTitle 		  = billetContainer.prev()
		;

		if ( checkoutValue == 'transparent_checkout' ) {
			publicKeyContainer.show();
			installmentsContainer.show();
			installmentsTitle.show();
			billetContainer.show();
			billetTitle.show();
		} else {
			publicKeyContainer.hide();
			installmentsContainer.hide();
			installmentsTitle.hide();
			billetContainer.hide();
			billetTitle.hide();
		}
	};

});
