<?php
namespace Woocommerce\Moip\Controller;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

//WooCommerce
use WC_Payment_Gateway;
use WC_Order;
use WC_Payment_Gateway_CC;

use Woocommerce\Moip\Core;
use Woocommerce\Moip\Model\Setting;
use Woocommerce\Moip\Helper\Utils;
use Woocommerce\Moip\Model\Custom_Gateway;
use Woocommerce\Moip\Model\Checkout;
use Woocommerce\Moip\Model\Order;
use Woocommerce\Moip\Model\Moip_SDK;

class Custom_Gateways extends WC_Payment_Gateway
{
	/**
	 * @var Object
	 */
	public $model;

	/**
	 * @var Object
	 */
	public $moip_order;

	public function __construct()
	{
		$this->model = new Custom_Gateway();

		$this->id                 = Core::SLUG;
		$this->method_title       = __( 'Moip Official', Core::SLUG );
		$this->method_description = __( 'Payment Gateway Moip', Core::SLUG );
		$this->has_fields         = false;
		$this->icon               = Core::plugins_url( 'assets/images/icons/logo.png' );

		$this->_set_webhook();
		$this->init_form_fields();
		$this->init_settings();
		$this->_check_errors();

		$this->enabled     = $this->get_option( 'enabled', 'no' );
		$this->title       = $this->get_option( 'title' );
		$this->description = $this->get_option( 'description' );

		if ( is_admin() ) {
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		}

		add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
		add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'thank_you_page' ) );
		add_action( 'admin_notices', array( $this, 'display_errors' ) );
	}

	/**
	 * Output the admin options table.
	 *
	 * @since 1.0
	 * @param null
	 * @return Void
	 */
	public function admin_options()
	{
		printf(
			'<h3>%s</h3>
			 <div %s>
			 	<table class="form-table">%s</table>
			 </div>',
			Core::get_name(),
			Utils::get_component( 'settings' ),
			$this->generate_settings_html( $this->get_form_fields(), false )
		);
	}

	/**
	 * Return the name of the option in the WP DB.
	 * @since 1.0
	 * @return string
	 */
	public function get_option_key()
	{
		return $this->model->settings->get_option_key();
	}

	public function is_available() {
		return ( $this->model->settings->is_enabled() && ! $this->get_errors() && $this->model->supported_currency() );
	}

	public function init_form_fields()
	{
		$this->form_fields = array(
			'auth'                     => $this->section_auth(),
			'section_settings'         => $this->section_settings(),
			'enabled'                  => $this->field_enabled(),
			'title'                    => $this->field_title(),
			'description'              => $this->field_description(),
			'invoice_name'             => $this->field_invoice_name(),
			'invoice_prefix'           => $this->field_invoice_prefix(),
			'section_payment_settings' => $this->section_payment_settings(),
			'public_key'               => $this->field_public_key(),
			'payment_api'              => $this->field_payment_type(),
			'billet_banking'           => $this->field_billet_banking(),
			'credit_card'              => $this->field_credit_card(),
			'banking_debit'            => $this->field_banking_debit(),
			'section_installments'     => $this->section_installments(),
			'installments_enabled'     => $this->field_credit_card_installments( 'enabled' ),
			'installments_maximum'     => $this->field_credit_card_installments( 'maximum' ),
			'installments'             => $this->field_credit_card_installments( 'general' ),
			'section_billet_settings'  => $this->section_billet_settings(),
			'billet_deadline_days'     => $this->field_billet_deadline_days(),
			'billet_instruction_line1' => $this->field_billet_instructions( 1 ),
			'billet_instruction_line2' => $this->field_billet_instructions( 2 ),
			'billet_instruction_line3' => $this->field_billet_instructions( 3 ),
			'billet_logo'              => $this->field_billet_logo_url(),
		);
	}

	public function section_auth()
	{
		return array(
			'title'       => __( 'APP authorization', Core::SLUG ),
			'type'        => 'title',
			'description' => $this->_get_app_status(),
		);
	}

	private function _get_app_status()
	{
		$class       = 'app-not-authorized';
		$status      = __( 'Not authorized', Core::SLUG );
		$description = __( 'To make your sales, you must authorize this application on Moip.', Core::SLUG );
		$via         = '';

		if ( $this->model->settings->authorize_data ) {
			$class  = 'app-authorized';
			$status = __( 'Authorized', Core::SLUG );
			$via    = sprintf( 'via %s', $this->model->settings->authorize_mode );
		}

		return sprintf(
			'%s<br><strong>Status: </strong><span class="%s">%s</span> %s %s',
			$description,
			$class,
			$status,
			$via,
			$this->_get_authorized_app_btn()
		);
	}

	public function section_payment_settings()
	{
		return array(
			'title' => __( 'Payment settings', Core::SLUG ),
			'type'  => 'title',
		);
	}

	public function section_settings()
	{
		return array(
			'title' => __( 'Settings', Core::SLUG ),
			'type'  => 'title',
		);
	}

	public function section_installments()
	{
		return array(
			'title' => __( 'Credit Card Installments Settings', Core::SLUG ),
			'type'  => 'title',
		);
	}

	public function section_billet_settings()
	{
		return array(
			'title' => __( 'Billet Settings', Core::SLUG ),
			'type'  => 'title',
		);
	}

	public function field_enabled()
	{
		return array(
			'title'   => __( 'Enable', Core::SLUG ),
			'type'    => 'checkbox',
			'label'   => __( 'Enable payment', Core::SLUG ),
			'default' => 'no',
		);
	}

	public function field_title()
	{
		return array(
			'title'       => __( 'Title', Core::SLUG ),
			'description' => __( 'This the title which the user sees during checkout.', Core::SLUG ),
			'desc_tip'    => true,
			'default'     => __( 'Moip', Core::SLUG ),
		);
	}

	public function field_description()
	{
		return array(
			'title'   => __( 'Description', Core::SLUG ),
			'default' => __( 'Pay with Moip', Core::SLUG ),
		);
	}

	public function field_invoice_name()
	{
		return array(
			'title'             => __( 'Invoice name', Core::SLUG ),
			'desc_tip'          => true,
			'placeholder'       => __( 'Maximum of 13 characters', Core::SLUG ),
			'description'       => __( 'It allows the shopkeeper to send a text of up to 13 characters that will be printed on the bearer\'s invoice, next to the shop identification, respecting the length of the flags.', Core::SLUG ),
			'custom_attributes' => array(
				'data-action'    => 'invoice-name',
				'data-element'   => 'validate',
				'maxlength'      => 13,
				'data-error-msg' => __( 'This field is required.', Core::SLUG ),
			),
		);
	}

	public function field_invoice_prefix()
	{
		return array(
			'title'       => __( 'Invoice prefix', Core::SLUG ),
			'default'     => 'WC-',
			'desc_tip'    => true,
			'description' => __( 'Enter a prefix for your invoice numbers. If you use your Moip account for multiple stores, make sure this prefix is unique because Moip will not allow orders with the same invoice number.', Core::SLUG ),
		);
	}

	public function field_payment_type()
	{
		return array(
			'type'              => 'select',
			'title'             => __( 'Payment Type', Core::SLUG ),
			'class'             => 'wc-enhanced-select',
			'default'           => 'transparent_checkout',
			'custom_attributes' => array(
				'data-element'  => 'checkout',
				'data-action'   => 'checkout-type',
			),
			'options' => array(
				'transparent_checkout' => __( 'Transparent Checkout', Core::SLUG ),
				'moip_checkout'        => __( 'Moip Checkout', Core::SLUG ),
			),
		);
	}

	public function field_public_key()
	{
		return array(
			'title'             => __( 'Public Key', Core::SLUG ),
			'type'              => 'textarea',
			'css'               => 'height: 200px',
			'custom_attributes' => array(
				'data-element'   => 'validate',
				'data-field'     => 'public-key',
				'data-error-msg' => __( 'This field is required.', Core::SLUG ),
			),
			'description' => __( 'Allows credit card data to be sent encrypted, generating more security in your transactions.', Core::SLUG )
		);
	}

	public function field_billet_banking()
	{
		return array(
			'title'   => __( 'Billet Banking', Core::SLUG ),
			'type'    => 'checkbox',
			'label'   => __( 'Enable Billet Banking', Core::SLUG ),
			'default' => 'yes',
		);
	}

	public function field_credit_card()
	{
		return array(
			'title'   => __( 'Credit Card', Core::SLUG ),
			'type'    => 'checkbox',
			'label'   => __( 'Enable Credit Card', Core::SLUG ),
			'default' => 'yes',
		);
	}

	public function field_banking_debit()
	{
		return array(
			'title'   => __( 'Banking Debit', Core::SLUG ),
			'type'    => 'checkbox',
			'label'   => __( 'Enable Banking Debit', Core::SLUG ),
			'default' => 'yes',
		);
	}

	public function field_billet_deadline_days()
	{
		return array(
			'title'             => __( 'Number of Days', Core::SLUG ),
			'description'       => __( 'Days of expiry of the billet after printed.', Core::SLUG ),
			'desc_tip'          => true,
			'placeholder'       => 5,
			'default'           => 5,
			'custom_attributes' => array(
				'data-field' => 'billet',
			),
		);
	}

	public function field_billet_instructions( $line )
	{
		$instructions = array(
			1 => array(
				'title'       => __( 'Instruction Line 1', Core::SLUG ),
				'type'        => 'text',
				'description' => __( 'First line instruction for the billet.', Core::SLUG ),
				'desc_tip'    => true,
			),
			2 => array(
				'title'       => __( 'Instruction Line 2', Core::SLUG ),
				'type'        => 'text',
				'description' => __( 'Second line instruction for the billet.', Core::SLUG ),
				'desc_tip'    => true,
			),
			3 => array(
				'title'       => __( 'Instruction Line 3', Core::SLUG ),
				'type'        => 'text',
				'description' => __( 'Third line instruction for the billet.', Core::SLUG ),
				'desc_tip'    => true,
			),
		);

		return $instructions[ $line ];
	}

	public function field_billet_logo_url()
	{
		return array(
			'title'       => __( 'Custom Logo URL', Core::SLUG ),
			'type'        => 'text',
			'description' => __( 'URL of the logo image to be shown on the billet.', Core::SLUG ),
			'desc_tip'    => true,
			'default'     => ''
		);
	}

	public function field_credit_card_installments( $field )
	{
		$installments = array();

		$installments['enabled'] = array(
			'title'             => __( 'Installments settings', Core::SLUG ),
			'type'              => 'checkbox',
			'label'             => __( 'Enable Installments settings', Core::SLUG ),
			'default'           => 'no',
			'custom_attributes' => array(
				'data-field' => 'installments',
			),
		);

		$installments['maximum'] = array(
			'title'       => __( 'Maximum installments number', Core::SLUG ),
			'type'        => 'select',
			'description' => __( 'Force a maximum number of installments for payment.', Core::SLUG ),
			'desc_tip'    => true,
			'default'     => 12,
			'options'     => $this->model->get_installment_options(),
			'custom_attributes' => array(
				'data-action' => 'installments-maximum',
			),
		);

		$installments['general'] = array(
			'title'       => __( 'Interest per installment', Core::SLUG ),
			'type'        => 'installments',
			'description' => __( 'Define interest for each installment.', Core::SLUG ),
			'desc_tip'    => true,
		);

		$installments['interest'] = array(
			'title'       => __( 'Interest', Core::SLUG ),
			'type'        => 'text',
			'description' => __( 'Interest to be applied to the installment.', Core::SLUG ),
			'desc_tip'    => true,
			'placeholder' => '0.00',
		);

		return $installments[ $field ];
	}

	public function process_payment( $order_id )
	{
		$order = new WC_Order( $order_id );

		return array(
			'result'   => 'success',
			'redirect' => $order->get_checkout_payment_url( true )
		);
	}

	public function receipt_page( $order_id )
	{
		if ( $this->model->settings->is_checkout_transparent() ) {
			$this->checkout_transparent( $order_id );
		} else {
			$this->checkout_moip( $order_id );
		}
	}

	public function thank_you_page( $order_id )
	{
		$order = new WC_Order( $order_id );

		require_once( Core::get_file_path( 'thank-you-page.php', 'templates/' ) );
	}

	public function checkout_moip( $order_id )
	{
		$order         = new Order( $order_id );
		$wc_order      = new WC_Order( $order_id );
		$payment_links = $order->payment_links;

		if ( ! is_object( $payment_links ) ) {
			$moip_sdk         = Moip_SDK::get_instance();
			$this->moip_order = $moip_sdk->create_order( $wc_order );

			if ( ! $this->moip_order ) {
				return;
			}

			$payment_links = $this->moip_order['response']->_links->checkout;
		}

		require_once( Core::get_file_path( 'checkout-moip.php', 'templates/' ) );

		unset( $order );
		unset( $wc_order );
	}

	public function checkout_transparent( $order_id )
	{
		$wc_order = new WC_Order( $order_id );

		require_once( Core::get_file_path( 'checkout-transparent.php', 'templates/' ) );
	}

	private function _set_webhook() {
		Webhooks::set_token( $this );
	}

	private function _check_errors()
	{
		if ( ! $this->model->settings->authorize_data ) {
			$this->add_error( __( 'Application not authorized.', Core::SLUG ) );
		}

		if ( ! $this->model->settings->invoice_name ) {
			$this->add_error( __( 'Invoice name is required.', Core::SLUG ) );
		}

		if ( ! $this->model->settings->public_key ) {
			$this->add_error( __( 'Not informed public key.', Core::SLUG ) );
		}

		return $this->errors;
	}

	public function display_errors()
	{
		if ( ! $this->get_errors() ) {
			return;
		}

		echo '<div id="woocommerce_errors" class="error notice is-dismissible">';

		printf(
			'<p><strong>%s:</strong> %s <a href="%s">%s</a></p>',
			Core::get_name(),
			__( 'You need to set up your Moip data to use the payment method.', Core::SLUG ),
			Core::get_page_link(),
			__( 'Go to setup', Core::SLUG )
		);

		echo '<ol>';

		foreach ( $this->get_errors() as $error ) {
			printf(
				'<li><strong>%s</strong></li> ',
				wp_kses_post( $error )
			);
		}

		echo '</ol>';
		echo '</div>';
	}

	/**
	 * Get HTML for descriptions.
	 *
	 * @param  array $data
	 * @return string
	 */
	public function get_description_html( $data )
	{
		if ( $data['desc_tip'] === true ) {
			return;
		} elseif ( ! empty( $data['desc_tip'] ) ) {
			$description = $data['description'];
		} elseif ( ! empty( $data['description'] ) ) {
			$description = $data['description'];
		} else {
			return;
		}

		return sprintf(
			'<p class="description %s">%s</p>',
			sanitize_html_class( Utils::get_value_by( $data, 'class_p' ) ),
			strip_tags( $description, '<a><span>' )
		);
	}

	public function generate_installments_html( $key, $data )
	{
		$field_key = $this->get_field_key( $key );
		$defaults  = array(
			'title'             => '',
			'disabled'          => false,
			'class'             => '',
			'css'               => '',
			'placeholder'       => '',
			'type'              => 'text',
			'desc_tip'          => false,
			'description'       => '',
			'custom_attributes' => array(),
		);

		$data  = wp_parse_args( $data, $defaults );
		$value = (array) $this->get_option( $key, array() );

		ob_start();

		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<?php echo $this->get_tooltip_html( $data ); ?>
				<label for="<?php echo esc_attr( $field_key ); ?>"><?php echo wp_kses_post( $data['title'] ); ?></label>
			</th>
			<td class="forminp">
				<fieldset>
					<?php
						for ( $i = 2; $i <= 12; $i++ ) :
							$interest = isset( $value['interest'][ $i ] ) ? $value['interest'][ $i ] : '';
					?>
					<p data-installment="<?php echo $i; ?>">
						<input class="small-input" type="text" value="<?php echo $i; ?>"
							   <?php disabled( 1, true ); ?> />
						<input class="small-input" type="text"
							   placeholder="0,00"
							   data-mask="##0,00" data-mask-reverse="true"
							   name="<?php echo esc_attr( $field_key ); ?>[interest][<?php echo $i; ?>]"
							   id="<?php echo esc_attr( $field_key ); ?>" value="<?php echo wc_format_localized_price( $interest ) ?>" />%
					</p>
					<?php endfor; ?>

					<?php echo $this->get_description_html( $data ); ?>
				</fieldset>
			</td>
		</tr>
		<?php

		return ob_get_clean();
	}

	public function validate_installments_field( $key, $value )
	{
		return $value;
	}

	private function _get_authorized_app_btn()
	{
		$title = __( 'Authorize App', Core::SLUG );
		$class = 'button-primary';

		if ( $this->model->settings->authorize_data ) {
			$title = __( 'New authorize', Core::SLUG );
			$class = '';
		}

		return sprintf(
			'<p>
				<a href="#"
				   class="button %s"
				   id="oauth-app-btn">
					%s
				</a>
			</p>',
			$class,
			$title
		);
	}
}
